#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.tui.factory
      @file: menu_option.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from typing import Any


class MenuOption:
    """TODO"""

    def __init__(self, parent: Any, option_index: int, option_text: str):
        self.parent = parent
        self.option_index = option_index
        self.option_text = option_text
        self.action_trigger = lambda s: print(f"Option: {self.option_index}-{self.option_text} selected!")

    def on_trigger(self, action_trigger) -> Any:
        """TODO"""

        self.action_trigger = action_trigger
        return self.parent

    def __str__(self) -> str:
        return f"%GREEN%[{self.option_index}]%NC% {self.option_text}"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.application
      @file: version.py
   @created: Thu, 14 Feb 2022
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""
import logging as log
import os
from typing import Tuple

from hspylib.core.tools.commons import read_version


class AppVersion:

    @staticmethod
    def load(filename: str = '.version', load_dir: str = os.getcwd()) -> 'AppVersion':
        filepath = f'{load_dir}/{filename}'
        if not os.path.exists(filepath):
            log.warning(f'File "{filepath}" does not exist')
            return AppVersion((0, 0, 0))

        return AppVersion(read_version(filepath))

    def __init__(self, version: Tuple[int, int, int]):
        self.version = version
        self.version_string = '.'.join(map(str, version))

    def __str__(self):
        return self.version_string

    def __repr__(self):
        return str(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.tui.extra.mdashboard
      @file: dashboard_builder.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""
from typing import Callable, List, Optional

from hspylib.modules.cli.icons.font_awesome.awesome import Awesome
from hspylib.modules.cli.tui.extra.mdashboard.dashboard_item import DashboardItem


class DashboardBuilder:
    """TODO"""

    @staticmethod
    class DashboardItemBuilder:
        """TODO"""

        def __init__(self, parent: 'DashboardBuilder'):
            self.parent = parent
            self.item = DashboardItem()

        def icon(self, icon: Awesome) -> 'DashboardBuilder.DashboardItemBuilder':
            """TODO"""
            self.item.icon = icon
            return self

        def tooltip(self, tooltip: str) -> 'DashboardBuilder.DashboardItemBuilder':
            """TODO"""
            self.item.tooltip = tooltip
            return self

        def on_trigger(self, on_trigger: Callable) -> Optional['DashboardBuilder.DashboardItemBuilder']:
            """TODO"""
            self.item.on_trigger = on_trigger
            return self

        def build(self) -> 'DashboardBuilder':
            """TODO"""
            self.parent.items.append(self.item)
            return self.parent

    def __init__(self):
        self.items = []

    def item(self) -> DashboardItemBuilder:
        """TODO"""
        return DashboardBuilder.DashboardItemBuilder(self)

    def build(self) -> List[DashboardItem]:
        """TODO"""
        return self.items



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.qt.promotions
      @file: hframe.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""
from typing import List, Optional, Union

from PyQt5.QtCore import pyqtSignal, Qt
from PyQt5.QtWidgets import QListWidget, QListWidgetItem, QWidget


class HListWidget(QListWidget):
    """TODO"""

    keyPressed = pyqtSignal(int)

    @staticmethod
    def unset_flag(flags: int, flag: int) -> int:
        """TODO"""
        return flags ^ flag if flags & flag == flag else flags

    def __init__(self, parent: Optional[QWidget] = None):
        super().__init__(parent)
        self.editable = False
        self.selectable = True
        self._items = []

    def keyPressEvent(self, event) -> None:
        """TODO"""
        super().keyPressEvent(event)
        self.keyPressed.emit(event.key())

    def addItem(self, item: QListWidgetItem) -> None:
        """TODO"""
        prev = self.item(self.count() - 1)
        if prev:
            item.setFlags(prev.flags())
        super().addItem(item)
        self._items.append(item)

    def set_item(self, item: str, flags: Union[Qt.ItemFlags, Qt.ItemFlag] = None) -> None:
        """TODO"""
        if not self.findItems(item, Qt.MatchFixedString):
            w_item = QListWidgetItem(item)
            w_item.setFlags(flags or Qt.ItemIsSelectable | Qt.ItemIsEditable | Qt.ItemIsEnabled)
            self.addItem(w_item)

    def del_item(self, index: int) -> None:
        """TODO"""
        if 0 < index < len(self._items):
            item = self.takeItem(index)
            if item:
                self._items.remove(self._items[index])

    def index_of(self, item: str) -> int:
        """TODO"""
        return next((obj for obj in self._items if obj == item), None)

    def as_list(self) -> List[str]:
        """TODO"""
        return list(map(QListWidgetItem.text, self._items))

    def text(self) -> str:
        """TODO"""
        return self.currentItem().text()

    def size(self) -> int:
        """TODO"""
        return self.count()

    def set_editable(self, editable: bool = True) -> None:
        """TODO"""
        for item in self._items:
            flags = int(item.flags())
            if editable:
                item.setFlags(flags | Qt.ItemIsEditable)
            else:
                item.setFlags(self.unset_flag(flags, int(Qt.ItemIsEditable)))
            self.editable = editable

    def set_selectable(self, selectable: bool = True) -> None:
        """TODO"""
        for item in self._items:
            flags = int(item.flags())
            if selectable:
                item.setFlags(flags | Qt.ItemIsSelectable)
            else:
                item.setFlags(self.unset_flag(flags, int(Qt.ItemIsSelectable)))
            self.selectable = selectable



# _*_ coding: utf-8 _*_
#
# hspylib v0.11.61
#
# Package: main.hspylib.modules.cli.tui.extra.minput
"""Package initialization."""

__all__ = [
  'access_type', 
  'field_builder', 
  'form_builder', 
  'form_field', 
  'input_type', 
  'input_validator', 
  'minput', 
  'minput_utils'
]
__version__ = '0.11.61'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   TODO Purpose of the file
   @project: HSPyLib
   main.addons.appman.templates
      @file: tpl-widget.py
   @created: Tue, 1 Jun 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""
import concurrent
from time import sleep
from typing import List

from hspylib.addons.widman.widget import Widget
from hspylib.core.enums.exit_code import ExitCode
from hspylib.core.tools.commons import sysout
from hspylib.modules.cli.icons.font_awesome.widget_icons import WidgetIcons
from hspylib.modules.cli.keyboard import Keyboard


class Widget_WIDGET_NAME_(Widget):
    """HSPyLib to do something"""
    WIDGET_ICON = WidgetIcons.WIDGET
    WIDGET_NAME = "_WIDGET_NAME_"
    TOOLTIP = "TODO Widget tooltip"
    USAGE = "Usage: _WIDGET_NAME_"
    VERSION = (0, 1, 0)

    def __init__(self):
        super().__init__(
            self.WIDGET_ICON,
            self.WIDGET_NAME,
            self.TOOLTIP,
            self.USAGE,
            self.VERSION)
        self._exit_code = ExitCode.SUCCESS

    def execute(self, args: List[str] = None) -> ExitCode:
        with concurrent.futures.ThreadPoolExecutor() as executor:
            done = False
            while not done and not Keyboard.kbhit():
                future = executor.submit(self._do_something)
                done = not future.result()
                sleep(0.5)

        return self._exit_code

    def cleanup(self) -> None:
        # If your widget requires any cleanup procedures
        pass

    def _do_something(self) -> None:
        sysout('')
        sysout('My widget is running')
        sysout('')



from typing import List

from PyQt5.QtCore import pyqtSlot, QAbstractAnimation, QEasingCurve, QParallelAnimationGroup, QPoint, \
    QPropertyAnimation, Qt
from PyQt5.QtWidgets import QStackedWidget, QWidget


class HStackedWidget(QStackedWidget):
    """TODO"""

    def __init__(self, parent=None):
        super(HStackedWidget, self).__init__(parent)
        self._slide_direction = Qt.Horizontal
        self._slide_speed = 500
        self._animation_type = QEasingCurve.OutCubic
        self._cur_idx = 0
        self._next_idx = 0
        self._wrap = False
        self._pos_current = QPoint(0, 0)
        self._widgets = []
        self._active = False

    def set_direction(self, direction) -> None:
        self._slide_direction = direction

    def set_speed(self, speed) -> None:
        self._slide_speed = speed

    def set_animation(self, animation_type) -> None:
        self._animation_type = animation_type

    def set_wrap(self, wrap) -> None:
        self._wrap = wrap

    def widgets(self) -> List[QWidget]:
        return self._widgets

    def addWidget(self, widget: QWidget) -> int:
        self._widgets.append(widget)
        return super().addWidget(widget)

    @pyqtSlot()
    def slide_previous(self) -> None:
        now = self.currentIndex()
        if self._wrap or now > 0:
            self.slide_to_index(now - 1)

    @pyqtSlot()
    def slide_next(self) -> None:
        now = self.currentIndex()
        if self._wrap or now < (self.count() - 1):
            self.slide_to_index(now + 1)

    def slide_to_index(self, idx: int) -> None:
        if idx != 0:
            if idx > (self.count() - 1):
                idx %= self.count()
            elif idx < 0:
                idx = (idx + self.count()) % self.count()
        self.slide_to_widget(self.widget(idx))

    def slide_to_widget(self, widget: QWidget) -> None:
        if self._active:
            return

        self._active = True

        idx = self.currentIndex()
        next_idx = self.indexOf(widget)

        if idx == next_idx:
            self._active = False
            return

        offset_x, offset_y = self.frameRect().width(), self.frameRect().height()
        self.widget(next_idx).setGeometry(self.frameRect())

        if not self._slide_direction == Qt.Horizontal:
            if idx < next_idx:
                offset_x, offset_y = 0, -offset_y
            else:
                offset_x = 0
        else:
            if idx < next_idx:
                offset_x, offset_y = -offset_x, 0
            else:
                offset_y = 0

        pos_next = self.widget(next_idx).pos()
        pos_current = self.widget(idx).pos()
        self._pos_current = pos_current

        offset = QPoint(offset_x, offset_y)
        self.widget(next_idx).move(pos_next - offset)
        self.widget(next_idx).show()
        self.widget(next_idx).raise_()

        anim_group = QParallelAnimationGroup(self, finished=self.animation_done)

        for index, start, end in zip(
            (idx, next_idx), (pos_current, pos_next - offset), (pos_current + offset, pos_next)
        ):
            animation = QPropertyAnimation(
                self.widget(index),
                b"pos",
                duration=self._slide_speed,
                easingCurve=self._animation_type,
                startValue=start,
                endValue=end,
            )
            anim_group.addAnimation(animation)

        self._next_idx = next_idx
        self._cur_idx = idx
        self._active = True
        anim_group.start(QAbstractAnimation.DeleteWhenStopped)

    @pyqtSlot()
    def animation_done(self) -> None:
        self.widget(self._cur_idx).hide()
        self.widget(self._cur_idx).move(self._pos_current)
        self._active = False
        self.setCurrentIndex(self._next_idx)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.qt.promotions
      @file: htableview.py
   @created: Wed, 30 Jun 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from typing import Optional

import pyperclip as clipboard
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QCursor, QPainter, QPaintEvent
from PyQt5.QtWidgets import QAbstractScrollArea, QHeaderView, QMenu, QTableView, QWidget


class HTableView(QTableView):
    """TODO"""

    def __init__(self, parent: Optional[QWidget], placeholder: Optional[str] = None):
        super().__init__(parent)
        self._context_menu_enable = True
        self._clearable = True
        self._deletable = True
        self._placeholder = placeholder or 'No data to display'
        self.customContextMenuRequested.connect(self._context_menu)
        self.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
        self.horizontalHeader().setStretchLastSection(True)
        self.verticalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
        self.setSizeAdjustPolicy(QAbstractScrollArea.AdjustToContents)

    def clear(self):
        """Clear the entire table"""
        model = self.model()
        if model:
            model.clear()

    def paintEvent(self, event: QPaintEvent) -> None:
        """TODO"""
        super().paintEvent(event)
        if self.model() is not None and self.model().rowCount() > 0:
            return
        color = self.palette().placeholderText().color()
        painter = QPainter(self.viewport())
        painter.save()
        painter.setPen(color)
        elided_text = self.fontMetrics() \
            .elidedText(self._placeholder, Qt.ElideRight, self.viewport().width())
        painter.drawText(self.viewport().rect(), Qt.AlignCenter, elided_text)
        painter.restore()

    def copy(self) -> None:
        """Copy selected cell into clipboard"""

        sel_model = self.selectionModel()
        if sel_model:
            index_list = sel_model.selectedIndexes()
            text = ''
            last_row = 0
            for index in index_list:
                if len(text) > 0:
                    if last_row == index.row():
                        text += ' '
                    else:
                        text += '\n'
                text += str(self.model().column(index))
                last_row = index.row()
            clipboard.copy(text)

    def delete(self) -> None:
        """Delete selected rows"""

        self.model().remove_rows(self.model().selected_rows()[0])

    def _context_menu(self):
        """Display the custom context menu"""
        if self._context_menu_enable:
            ctx_menu = QMenu(self)
            ctx_menu.addAction('Copy', self.copy)
            if self._deletable:
                ctx_menu.addAction('Delete', self.delete)
            if self._clearable:
                ctx_menu.addSeparator()
                ctx_menu.addAction('Clear', self.clear)
            ctx_menu.exec_(QCursor.pos())

    def set_context_menu_enable(self, enabled: bool = True):
        """Whether context menu is enabled or not"""
        self._context_menu_enable = enabled

    def set_clearable(self, clearable: bool = True):
        """Whether the widget is clearable or not"""
        self._clearable = clearable

    def set_deletable(self, deletable: bool = True):
        """Whether the widget is clearable or not"""
        self._deletable = deletable



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: main.modules.cli.tui.extra
      @file: mselect.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""
import re
from abc import ABC
from typing import List, Optional, TypeVar

from hspylib.core.tools.commons import sysout
from hspylib.modules.cli.icons.font_awesome.form_icons import FormIcons
from hspylib.modules.cli.keyboard import Keyboard
from hspylib.modules.cli.vt100.vt_codes import vt_print
from hspylib.modules.cli.vt100.vt_colors import VtColors
from hspylib.modules.cli.vt100.vt_utils import prepare_render, restore_cursor, restore_terminal, screen_size

T = TypeVar('T')


def mselect(
    items: List[T],
    title: str = 'Please select one',
    max_rows: int = 15,
    title_color: VtColors = VtColors.ORANGE,
    highlight_color: VtColors = VtColors.BLUE,
    nav_color: VtColors = VtColors.YELLOW) -> Optional[T]:
    """
    TODO
    :param items:
    :param title:
    :param max_rows:
    :param title_color:
    :param highlight_color:
    :param nav_color:
    :return:
    """
    return MenuSelect(items, max_rows).select(title, title_color, highlight_color, nav_color)


class MenuSelect(ABC):
    """TODO"""
    UNSELECTED = ' '
    SELECTED = FormIcons.SELECTOR

    NAV_ICONS = '\u2191\u2193'
    NAV_FMT = "\n{}[Enter] Select  [{}] Navigate  [Q] Quit  [1..{}] Goto: %EL0%"

    def __init__(
        self,
        items: List[T],
        max_rows: int = 15):

        self.items = items
        self.show_from = 0
        self.show_to = max_rows - 1
        self.diff_index = self.show_to - self.show_from
        self.sel_index = 0
        self.re_render = True
        self.done = None

    def select(
        self,
        title: str,
        title_color: VtColors,
        highlight_color: VtColors,
        nav_color: VtColors) -> Optional[T]:
        """TODO"""

        ret_val = Keyboard.VK_NONE
        length = len(self.items)

        if length == 0:
            return None
        elif length == 1:  # When only one option is provided, select the element at index 0 and return
            return self.items[0]

        prepare_render(title, title_color)

        # Wait for user interaction
        while not self.done and ret_val not in [Keyboard.VK_Q, Keyboard.VK_q, Keyboard.VK_ENTER, Keyboard.VK_ESC]:
            # Menu Renderization
            if self.re_render:
                self._render(highlight_color, nav_color)

            # Navigation input
            ret_val = self._nav_input()

        restore_terminal()

        return self.items[self.sel_index] if ret_val == Keyboard.VK_ENTER else None

    def _render(self, highlight_color: VtColors, nav_color: VtColors) -> None:
        """TODO"""

        length = len(self.items)
        dummy, columns = screen_size()
        restore_cursor()

        for idx in range(self.show_from, self.show_to):
            selector = self.UNSELECTED

            if idx < length:  # When the number of items is lower than the max_rows, skip the other lines
                option_line = str(self.items[idx])[0:int(columns)]
                vt_print('%EL2%\r')  # Erase current line before repaint

                # Print the selector if the index is currently selected
                if idx == self.sel_index:
                    vt_print(highlight_color.code())
                    selector = self.SELECTED

                fmt = "  {:>" + str(len(str(length))) + "}{:>" + str(1 + len(str(selector))) + "} {}"
                sysout(fmt.format(idx + 1, selector, option_line))

                # Check if the text fits the screen and print it, otherwise print '...'
                if len(option_line) >= int(columns):
                    sysout("%CUB(4)%%EL0%...%NC%", end='')
            else:
                break

        sysout(self.NAV_FMT.format(nav_color.placeholder(), self.NAV_ICONS, str(length)), end='')
        self.re_render = False

    # pylint: disable=too-many-branches
    def _nav_input(self) -> chr:
        """TODO"""

        length = len(self.items)
        keypress = Keyboard.read_keystroke()

        if keypress:
            if keypress in [Keyboard.VK_q, Keyboard.VK_Q, Keyboard.VK_ESC]:
                self.done = True
            elif keypress.isdigit():  # An index was typed
                typed_index = keypress.value
                sysout(f"{keypress.value}", end='')
                index_len = 1
                while len(typed_index) < len(str(length)):
                    numpress = Keyboard.read_keystroke()
                    if not numpress:
                        break
                    if not re.match(r'^[0-9]*$', numpress.value):
                        typed_index = None
                        break
                    typed_index = f"{typed_index}{numpress.value if numpress else ''}"
                    sysout(f"{numpress.value if numpress else ''}", end='')
                    index_len += 1
                # Erase the index typed by the user
                sysout(f"%CUB({index_len})%%EL0%", end='')
                if 1 <= int(typed_index) <= length:
                    self.show_to = max(int(typed_index), self.diff_index)
                    self.show_from = self.show_to - self.diff_index
                    self.sel_index = int(typed_index) - 1
                    self.re_render = True
            elif keypress == Keyboard.VK_UP:  # Cursor up
                if self.sel_index == self.show_from and self.show_from > 0:
                    self.show_from -= 1
                    self.show_to -= 1
                if self.sel_index - 1 >= 0:
                    self.sel_index -= 1
                    self.re_render = True
            elif keypress == Keyboard.VK_DOWN:  # Cursor down
                if self.sel_index + 1 == self.show_to and self.show_to < length:
                    self.show_from += 1
                    self.show_to += 1
                if self.sel_index + 1 < length:
                    self.sel_index += 1
                    self.re_render = True
            elif keypress == Keyboard.VK_ENTER:  # Enter
                self.done = True

        self.re_render = True
        return keypress
